"use strict";

var _expect = require("expect");

var _expect2 = _interopRequireDefault(_expect);

var _fluxStandardAction = require("flux-standard-action");

var _actions = require("../actions");

var _actions2 = _interopRequireDefault(_actions);

var _actionTypes = require("../actionTypes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var receivePage = _actions2.default.receivePage,
    requestPage = _actions2.default.requestPage,
    resetView = _actions2.default.resetView,
    changeView = _actions2.default.changeView; /* eslint-disable no-undefined, max-lines-per-function, no-magic-numbers */

describe("actions", function () {

  it("should create receive page action", function () {
    var action = receivePage({
      endpoint: "some/api/endpoint/",
      pageArgName: "p",
      idKey: "id",
      page: 2,
      token: "foo=bar",
      items: ["foo", "bar", "baz"],
      total: 42
    });

    (0, _expect2.default)(action).toEqual({
      type: _actionTypes.RECEIVE_PAGE,
      meta: {
        endpoint: "some/api/endpoint/",
        pageArgName: "p",
        idKey: "id"
      },
      payload: {
        token: "foo=bar",
        page: 2,
        items: ["foo", "bar", "baz"],
        total: 42
      }
    });

    (0, _expect2.default)((0, _fluxStandardAction.isFSA)(action)).toEqual(true);
  });

  it("should create request page action", function () {
    var action = requestPage({
      endpoint: "some/api/endpoint/",
      resultsKey: "results",
      totalKey: "total",
      pageArgName: "p",
      idKey: "id",
      page: 2,
      token: "foo=bar"
    });

    (0, _expect2.default)(action).toEqual({
      type: _actionTypes.REQUEST_PAGE,
      meta: {
        endpoint: "some/api/endpoint/",
        resultsKey: "results",
        totalKey: "total",
        pageArgName: "p",
        idKey: "id"
      },
      payload: {
        token: "foo=bar",
        page: 2
      }
    });

    (0, _expect2.default)((0, _fluxStandardAction.isFSA)(action)).toEqual(true);
  });

  it("should create reset view action", function () {
    var action = resetView("some/api/endpoint/", "name");

    (0, _expect2.default)(action).toEqual({
      type: _actionTypes.RESET_VIEW,
      meta: {
        endpoint: "some/api/endpoint/"
      },
      payload: {
        token: "name"
      }
    });

    (0, _expect2.default)((0, _fluxStandardAction.isFSA)(action)).toEqual(true);
  });

  it("should create change view action", function () {
    var action = changeView("some/api/endpoint/", {
      token: "name",
      view: 2
    });

    (0, _expect2.default)(action).toEqual({
      type: _actionTypes.CHANGE_VIEW,
      meta: {
        endpoint: "some/api/endpoint/"
      },
      payload: {
        view: 2,
        token: "name"
      }
    });

    (0, _expect2.default)((0, _fluxStandardAction.isFSA)(action)).toEqual(true);
  });
});