"use strict";

var _immutable = require("immutable");

var Immutable = _interopRequireWildcard(_immutable);

var _chai = require("chai");

var _chai2 = _interopRequireDefault(_chai);

var _chaiImmutable = require("chai-immutable");

var _chaiImmutable2 = _interopRequireDefault(_chaiImmutable);

var _actions = require("../actions");

var _actions2 = _interopRequireDefault(_actions);

var _reducers = require("../reducers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

_chai2.default.use(_chaiImmutable2.default); /* eslint-disable max-len, no-undefined, max-lines-per-function */

var requestPage = _actions2.default.requestPage,
    receivePage = _actions2.default.receivePage;


var requestPageAction = requestPage({
  endpoint: "some/api/endpoint",
  resultsKey: "results",
  totalKey: "total",
  pageArgName: "p",
  idKey: "id",
  page: 2,
  token: "foo=bar"
});

var receivePageAction = receivePage({
  endpoint: "some/api/endpoint/",
  pageArgName: "p",
  idKey: "id",
  page: 2,
  token: "foo=bar",
  items: [{
    id: "baz",
    fooField: "bazValue"
  }, {
    id: "bar",
    fooField: "barValue"
  }],
  total: 2
});

describe("pages reducer", function () {

  it("should return the state by default", function () {
    var state = (0, _reducers.pages)(Immutable.Map({
      some: "state"
    }), { type: "some action" });

    (0, _chai.expect)(state).to.equal(Immutable.Map({
      some: "state"
    }));
  });

  it("should initialize the pages map with a new page entry when requesting new page", function () {
    var state = (0, _reducers.pages)(undefined, requestPageAction);

    (0, _chai.expect)(state).to.equal(Immutable.Map({
      "foo=bar": Immutable.Map({
        page: 2,
        token: "foo=bar",
        ids: Immutable.List(),
        fetching: true,
        error: false,
        view: 1,
        fetched: false
      })
    }));
  });

  it("should populate the pages map at the page url key with the item ids", function () {
    var state = (0, _reducers.pages)(Immutable.Map({
      "foo=bar": Immutable.Map({
        page: 2,
        token: "foo=bar",
        ids: Immutable.List(),
        fetching: true,
        fetched: false
      })
    }), receivePageAction);

    (0, _chai.expect)(state).to.equal(Immutable.Map({
      "foo=bar": Immutable.Map({
        page: 2,
        token: "foo=bar",
        ids: Immutable.List(["baz", "bar"]),
        fetching: false,
        fetched: true,
        error: false,
        total: 2
      })
    }));
  });
});

describe("items reducer", function () {

  it("should populate the items map with item received", function () {
    var state = (0, _reducers.items)(undefined, receivePageAction);

    (0, _chai.expect)(state).to.equal(Immutable.Map({
      "baz": Immutable.Map({
        id: "baz",
        fooField: "bazValue"
      }),
      "bar": Immutable.Map({
        id: "bar",
        fooField: "barValue"
      })
    }));
  });
});