"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reselect = require("reselect");

var _immutable = require("immutable");

var Immutable = _interopRequireWildcard(_immutable);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

/*
  Use to create a complex list where you need more information about
  the item. For instance, if you DO need to fetch more data or to
  manipulate it
*/
var createDataReducer = function createDataReducer(_ref) {
  var key = _ref.key;

  var getItems = function getItems(state) {
    return state.getIn(["entities", key]);
  },
      getItem = (0, _reselect.createSelector)(getItems, function (state, id) {
    return id;
  }, function (data, id) {
    return data.get(id) || Immutable.Map();
  }),
      getItemsList = (0, _reselect.createSelector)(getItems, function (data) {
    return data.map(function (current) {
      return current.get("Data");
    }).toList();
  }),
      getItemData = (0, _reselect.createSelector)(getItems, function (state, id) {
    return id;
  }, function (data, id) {
    if (typeof data === "undefined") {
      return Immutable.Map();
    }

    return data.getIn([id, "Data"]);
  }),
      getItemHasError = (0, _reselect.createSelector)(getItem, function (data) {
    if (typeof data === "undefined") {
      return false;
    }

    return data.get("error") === true;
  }),
      getIsFetchingItemInfo = (0, _reselect.createSelector)(getItem, function (data) {
    if (typeof data === "undefined") {
      return false;
    }

    return data.get("fetching");
  }),
      getItemIsFetched = (0, _reselect.createSelector)(getItem, function (data) {
    if (typeof data === "undefined") {
      return false;
    }

    return data.get("fetched");
  }),
      getShouldFetchItemInfo = (0, _reselect.createSelector)(getIsFetchingItemInfo, getItemIsFetched, function (isFetching, isFetched) {
    return !isFetched && !isFetching;
  });

  return {
    getItems: getItems,
    getItem: getItem,
    getItemsList: getItemsList,
    getItemData: getItemData,
    getItemHasError: getItemHasError,
    getIsFetchingItemInfo: getIsFetchingItemInfo,
    getItemIsFetched: getItemIsFetched,
    getShouldFetchItemInfo: getShouldFetchItemInfo
  };
};
/* eslint-disable max-lines-per-function,  */

exports.default = createDataReducer;