"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dataItems = exports.items = exports.pages = undefined;

var _actionTypes = require("./actionTypes");

var _immutable = require("immutable");

var Immutable = _interopRequireWildcard(_immutable);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var requestPage = function requestPage(state, action) {
  var _action$payload = action.payload,
      token = _action$payload.token,
      page = _action$payload.page;


  var elements = Immutable.Map({
    page: page,
    error: false,
    fetching: true,
    fetched: false
  });

  if (state.has(token)) {
    return state.update(token, function (current) {
      return current.merge(elements);
    });
  }

  var init = elements.merge({
    view: 1,
    token: token,
    ids: Immutable.List()
  });

  return state.set(token, init);
},
    receivePage = function receivePage(state, action) {
  var idKey = action.meta.idKey,
      _action$payload2 = action.payload,
      token = _action$payload2.token,
      items = _action$payload2.items,
      total = _action$payload2.total,
      error = _action$payload2.error;


  if (state.has(token)) {
    var hasError = error === true;

    return state.update(token, function (current) {
      return current.update(function (page) {
        return current.merge(Immutable.Map({
          ids: page.get("ids").concat(Immutable.List(items.map(function (item) {
            return String(item[idKey]);
          }))),
          fetching: false,
          error: hasError,
          fetched: true,
          total: hasError ? page.get("total") : total
        }));
      });
    });
  }

  return state;
},
    performChangeView = function performChangeView(state, action, view) {
  var token = action.payload.token;


  var item = state.get(token);

  if (typeof item === "undefined") {
    return state;
  }

  return state.update(token, function (current) {
    return current.set("view", view);
  });
},
    changeView = function changeView(state, action) {
  return performChangeView(state, action, action.payload.view);
},
    resetView = function resetView(state, action) {
  return performChangeView(state, action, 1);
},
    receivePageItems = function receivePageItems(state, action) {
  var payload = action.payload,
      meta = action.meta;
  var payloadItems = payload.items;
  var idKey = meta.idKey,
      manageEntity = meta.manageEntity;


  var manage = typeof manageEntity === "function" ? manageEntity : function (item) {
    return Immutable.Map(item);
  };

  var newItems = payloadItems.reduce(function (previous, item) {
    return previous.set(String(item[idKey]), manage(item, state, idKey));
  }, Immutable.Map());

  return state.merge(newItems);
};

var pages = exports.pages = function pages() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Immutable.Map();
  var action = arguments[1];

  switch (action.type) {
    case _actionTypes.REQUEST_PAGE:
      return requestPage(state, action);

    case _actionTypes.RECEIVE_PAGE:
      return receivePage(state, action);

    case _actionTypes.RESET_VIEW:
      return resetView(state, action);

    case _actionTypes.CHANGE_VIEW:
      return changeView(state, action);

    case _actionTypes.FETCH_CURRENT_COMPANY_INFO_PENDING:
    case _actionTypes.CLEAR_DATA:
      return state.clear();

    default:
      return state;
  }
};

var items = exports.items = function items() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Immutable.Map();
  var action = arguments[1];

  switch (action.type) {
    case _actionTypes.RECEIVE_PAGE:
      return receivePageItems(state, action);

    case _actionTypes.FETCH_CURRENT_COMPANY_INFO_PENDING:
    case _actionTypes.CLEAR_DATA:
      return state.clear();
    default:
      return state;
  }
};

// ---------- data items reducer

var fetchItemDataPending = function fetchItemDataPending(state, _ref) {
  var id = _ref.meta.id;
  return state.update(id, function (current) {
    if (typeof current === "undefined") {
      return current;
    }

    return current.mergeDeep(Immutable.Map({
      fetching: true,
      fetched: false,
      error: false
    }));
  });
},
    fetchItemDataRejected = function fetchItemDataRejected(state, _ref2) {
  var id = _ref2.meta.id;
  return state.update(id, function (current) {
    if (typeof current === "undefined") {
      return current;
    }

    return current.mergeDeep(Immutable.Map({
      fetching: false,
      fetched: false,
      error: true
    }));
  });
},
    fetchItemDataFulFilled = function fetchItemDataFulFilled(state, _ref3) {
  var Data = _ref3.payload.Data,
      id = _ref3.meta.id;
  return state.update(id, function (current) {
    if (typeof current === "undefined") {
      return current;
    }

    return current.mergeDeep(Immutable.Map({
      Data: Data,
      fetching: false,
      fetched: true,
      error: false
    }));
  });
};

var dataItems = exports.dataItems = function dataItems() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Immutable.Map();
  var action = arguments[1];

  switch (action.type) {
    case _actionTypes.RECEIVE_PAGE:
      return receivePageItems(state, action);

    case _actionTypes.FETCH_ITEM_DATA_PENDING:
      return fetchItemDataPending(state, action);
    case _actionTypes.FETCH_ITEM_DATA_REJECTED:
      return fetchItemDataRejected(state, action);
    case _actionTypes.FETCH_ITEM_DATA_FULFILLED:
      return fetchItemDataFulFilled(state, action);

    case _actionTypes.FETCH_CURRENT_COMPANY_INFO_PENDING:
    case _actionTypes.CLEAR_DATA:
      return state.clear();
    default:
      return state;
  }
};