"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _immutable = require("immutable");

var Immutable = _interopRequireWildcard(_immutable);

var _reselect = require("reselect");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var pagesSelector = function pagesSelector(state) {
  return state;
};
/* eslint-disable no-undefined */

var rowsPerPage = 50;

var pageSelector = (0, _reselect.createSelector)(pagesSelector, function (state, token) {
  return token;
}, function (pages, token) {
  return pages.get(token);
});

var getAllResults = (0, _reselect.createSelector)(pageSelector, function (state, token, items) {
  return items;
}, function (page, items) {
  if (typeof page === "undefined") {
    return Immutable.List();
  }

  return page.get("ids").map(function (id) {
    return items.get(String(id));
  });
});

var getResultsUpToPage = (0, _reselect.createSelector)(pageSelector, function (state, token, items) {
  return items;
}, function (state, token, items, target) {
  return target;
}, function (state, token, items, target, perPage) {
  return perPage || rowsPerPage;
}, function (page, items, target, perPage) {
  if (typeof page === "undefined") {
    return Immutable.List();
  }

  var ids = page.get("ids").slice(0, perPage * target);

  return ids.map(function (id) {
    return items.get(String(id));
  });
});

var getCurrentTotalResultsCount = (0, _reselect.createSelector)(pageSelector, function (page) {
  if (typeof page === "undefined") {
    return 0;
  }

  return page.get("total");
});

var isPageFetching = (0, _reselect.createSelector)(pageSelector, function (state, token, target) {
  return target;
}, function (page, target) {
  if (typeof page === "undefined") {
    return false;
  }

  return page.get("fetching") && target === page.get("page");
});

var isPageFetched = (0, _reselect.createSelector)(pageSelector, function (state, token, target) {
  return target;
}, function (page, target) {
  if (typeof page === "undefined") {
    return false;
  }

  return page.get("fetched") && target <= page.get("page");
});

var hasPageProblems = (0, _reselect.createSelector)(pageSelector, function (page) {
  if (typeof page === "undefined") {
    return false;
  }

  return page.get("error");
});

var getCurrentView = (0, _reselect.createSelector)(pageSelector, function (page) {
  if (typeof page === "undefined") {
    return 1;
  }

  return page.get("view");
});

// these selectors are used for the entire app
var getPaginatorPaginations = function getPaginatorPaginations(state, name) {
  return state.getIn(["paginations", name]);
},
    getPaginatorEntities = function getPaginatorEntities(state, name) {
  return state.getIn(["entities", name]);
},
    getPaginators = (0, _reselect.createSelector)(getPaginatorPaginations, getPaginatorEntities, function (list, entities) {
  return {
    list: list,
    entities: entities
  };
});

exports.default = {
  getAllResults: getAllResults,
  getCurrentTotalResultsCount: getCurrentTotalResultsCount,
  getResultsUpToPage: getResultsUpToPage,
  isPageFetching: isPageFetching,
  isPageFetched: isPageFetched,
  hasPageProblems: hasPageProblems,
  getCurrentView: getCurrentView,

  // for state
  getPaginatorPaginations: getPaginatorPaginations,
  getPaginatorEntities: getPaginatorEntities,
  getPaginators: getPaginators
};